@@ -4,32 +4,32 @@ use DBI;
 
 use FindBin qw($Bin);
 use IO::Socket::INET;
+use MogileFS::Server;
 
-sub create_temp_db {
-    my $dbname = "tmp_mogiletest";
-    create_mysql_db($dbname);
-    return DBHandle->new($dbname);
+sub temp_store {
+    my $type = $ENV{MOGTEST_DBTYPE};
+
+    # default to mysql, but make sure DBD::MySQL is installed
+    unless ($type) {
+        $type = "MySQL";
+        eval "use DBD::mysql; 1" or
+            die "DBD::mysql isn't installed.  Please install it or define MOGTEST_DBTYPE env. variable";
+    }
+
+    die "Bogus type" unless $type =~ /^\w+$/;
+    my $store = "MogileFS::Store::$type";
+    eval "use $store; 1;";
+    if ($@) {
+        die "Failed to load $store: $@\n";
+    }
+    my $sto = $store->new_temp;
+    Mgd::set_store($sto);
+    return $sto;
 }
 
-my $rootdbh;
-sub root_dbh {
-    return $rootdbh ||= DBI->connect("DBI:mysql:mysql", "root", "", { RaiseError => 1 })
-        or die "Couldn't connect to database";
-}
-
-sub create_mysql_db {
-    my $dbname = shift;
-    drop_mysql_db($dbname);
-    root_dbh()->do("CREATE DATABASE $dbname");
-}
-
-sub drop_mysql_db {
-    my $dbname = shift;
-    root_dbh()->do("DROP DATABASE IF EXISTS $dbname");
-}
 
 sub create_temp_tracker {
-    my $db = shift;
+    my $sto = shift;
 
     my $pid = fork();
     my $whoami = `whoami`;
@@ -48,9 +48,9 @@ sub create_temp_tracker {
              ($whoami eq "root" ? "--user=root" : ()),
              "--skipconfig",
              "--workers=2",
-             "--dsn=" . $db->dsn,
-             "--dbuser=" . $db->user,
-             "--dbpass=" . $db->pass);
+             "--dsn=" . $sto->dsn,
+             "--dbuser=" . $sto->user,
+             "--dbpass=" . $sto->pass);
     }
 
     for (1..3) {
@@ -92,34 +92,6 @@ sub create_mogstored {
     return undef;
 }
 
-############################################################################
-package DBHandle;
-sub new {
-    my ($class, $name) = @_;
-    return bless { name => $name }, $class;
-}
-
-sub dbh {
-    my $self = shift;
-    return $self->{dbh} ||= DBI->connect($self->dsn, $self->user, $self->pass, { RaiseError => 1 })
-        or die "Couldn't connect to '$self->{name}' database";
-}
-
-sub name {
-    my $self = shift;
-    return $self->{name};
-}
-
-sub dsn {
-    my $self = shift;
-    return "DBI:mysql:$self->{name}";
-}
-
-*username = \&user;
-sub user { "root" }
-*password = \&pass;
-sub pass { "" }
-
 ############################################################################
 package ProcessHandle;
 sub new {
